import tkinter as tk
import numpy as np
import cv2
from PIL import Image, ImageTk
import platform
import os

class EasyTk:
    def __init__(self, title="APP", size=None):
        """
        初始化EasyTk窗口
        :param title: 窗口标题
        :param size: 窗口大小（格式："宽度x高度"，例如"480x800"）
        """
        self.root = tk.Tk()
        self.root.title(title)
        self.root.geometry(size)

        if platform.system() == "Linux":  # 灵芯派默认设置
            if size is None:
                self.root.geometry("480x800")  # 窗口分辨率480*800
            self.root.resizable(False, False)  # 固定窗口大小
            self.root.attributes('-fullscreen', True)  # 全屏
            self.root.config(cursor="none")  # 隐藏鼠标
        
        # self.root.update_idletasks()  # 强制立即应用全屏，避免闪烁/延迟

        self._image = None


    def add_frame(self, master=None, **kwargs):
        """
        添加一个框架（LabelFrame）到窗口中
    
        :param master: 父容器（默认是root）
        :param kwargs: 组件配置和布局参数
            组件配置常用参数：
            - width: 宽度（像素）,如果为空则根据内容自适应
            - height: 高度（像素），如果为空则根据内容自适应
            - bg: 背景颜色，例如"white"
            组件布局常用参数：
            - side: 组件在父容器中的位置，tk.TOP（上）、tk.BOTTOM（下）、tk.LEFT（左）、tk.RIGHT（右）
            - fill: 组件在父容器中的填充方式，tk.BOTH（填充父容器）、tk.X（填充水平方向）、tk.Y（填充垂直方向）
            - expand: 是否允许组件在父容器中扩展，True（允许）、False（不允许）
            - anchor: 组件在父容器中的对齐方式，tk.N（北）、tk.S（南）、tk.E（东）、tk.W（西）、tk.CENTER（居中）
            - padx: 组件在父容器中的水平间距，单位像素
            - pady: 组件在父容器中的垂直间距，单位像素

        :return: 新添加的框架（LabelFrame）
        :rtype: LabelFrame
        """
        kwargs['master'] = master if master else self.root
        pack_kwargs = {
            'side': kwargs.pop('side', tk.TOP),
            'fill': kwargs.pop('fill', tk.BOTH),
            'expand': kwargs.pop('expand', False if kwargs['master'] == self.root else True),
            'anchor': kwargs.pop('anchor', tk.CENTER),
            'padx': kwargs.pop('padx', 0),
            'pady': kwargs.pop('pady', 0)}

        frame = tk.Frame(**kwargs)
        if kwargs.get('width', None) and kwargs.get('height', None):
            pack_kwargs['fill'] = tk.NONE
        elif kwargs.get('width', None):
            pack_kwargs['fill'] = tk.Y
        elif kwargs.get('height', None):
            pack_kwargs['fill'] = tk.X

        frame.pack_propagate(False if (kwargs.get('width', None) or kwargs.get('height', None)) else True)
        frame.pack(**pack_kwargs)

        return frame

    def add_label(self, master=None, **kwargs):
        """
        添加一个标签（Label）到窗口中, 可以显示文本或图片
    
        :param master: 父容器（默认是root）
        :param kwargs: 组件配置和布局参数
            组件配置常用参数：
            - text: 显示的文本内容
            - image: 显示的图片路径、opencv图像（np.array）、PIL.Image.Image对象
            - font: 字体设置，例如("SimHei", 24)
            - fg: 字体颜色，例如"red"
            - bg: 背景颜色，例如"white"
            - width: 宽度（像素）,如果为空则根据内容自适应
            - height: 高度（像素），如果为空则根据内容自适应
            - wraplength: 文本换行长度，单位像素（默认0，不换行）
            组件布局常用参数：
            - side: 组件在父容器中的位置，tk.TOP（上）、tk.BOTTOM（下）、tk.LEFT（左）、tk.RIGHT（右）
            - fill: 组件在父容器中的填充方式，tk.BOTH（填充父容器）、tk.X（填充水平方向）、tk.Y（填充垂直方向）
            - expand: 是否允许组件在父容器中扩展，True（允许）、False（不允许）
            - anchor: 组件在父容器中的对齐方式，tk.N（北）、tk.S（南）、tk.E（东）、tk.W（西）、tk.CENTER（居中）
            - padx: 组件在父容器中的水平间距，单位像素
            - pady: 组件在父容器中的垂直间距，单位像素
        :return: 新添加的标签（Label）
        :rtype: Label
        """
        # 在Label外部添加一个Frame，用于布局控制
        master = master if master else self.root
        _kwargs = {
            "side" : kwargs.pop('side', tk.TOP),  # 堆叠方向
            "fill" : kwargs.pop('fill', tk.BOTH),  # 填充方向
            "expand" : kwargs.pop('expand', False if master == self.root else True) ,  # 是否展开填充
            "anchor" : kwargs.pop('anchor', tk.CENTER),
            "padx" : kwargs.pop('padx', 0),
            "pady" : kwargs.pop('pady', 0),
            "width" : kwargs.pop('width', None),
            "height" : kwargs.pop('height', None),
            "bg": kwargs.get('bg', None),
        }
        _frame = self.add_frame(master, **_kwargs)
        
        # 在Frame内部创建Label组件
        image = kwargs.pop('image', None)
        label = tk.Label(_frame, **kwargs)
        if image is not None:  # 配置图片
            self.config(label, image=image)
        label.pack(fill=tk.BOTH, expand=True)  # 放置组件
        return label
    

    def add_button(self, master=None, **kwargs):
        """
        添加一个按钮（Button）到窗口中, 可以显示文本或图片

        :param master: 父容器（默认是root）
        :param kwargs: 组件配置和布局参数
            组件配置常用参数：
            - text: 显示的文本内容
            - image: 显示的图片路径、opencv图像（np.array）、PIL.Image.Image对象
            - font: 字体设置，例如("SimHei", 24)
            - fg: 字体颜色，例如"red"
            - bg: 背景颜色，例如"white"
            - width: 宽度（像素）,如果为空则根据内容自适应
            - height: 高度（像素），如果为空则根据内容自适应
            - wraplength: 文本换行长度，单位像素（默认0，不换行）
            组件布局常用参数：
            - side: 组件在父容器中的位置，tk.TOP（上）、tk.BOTTOM（下）、tk.LEFT（左）、tk.RIGHT（右）
            - fill: 组件在父容器中的填充方式，tk.BOTH（填充父容器）、tk.X（填充水平方向）、tk.Y（填充垂直方向）
            - expand: 是否允许组件在父容器中扩展，True（允许）、False（不允许）
            - anchor: 组件在父容器中的对齐方式，tk.N（北）、tk.S（南）、tk.E（东）、tk.W（西）、tk.CENTER（居中）
            - padx: 组件在父容器中的水平间距，单位像素
            - pady: 组件在父容器中的垂直间距，单位像素
        :return: 新添加的按钮（Button）
        :rtype: Button
        """

        kwargs['master'] = master if master else self.root
        pack_kwargs = {
            'side': kwargs.pop('side', tk.TOP),
            'fill': kwargs.pop('fill', tk.NONE),  # 按钮默认不填充父容器
            'expand': kwargs.pop('expand', False if kwargs['master'] == self.root else True),
            'anchor': kwargs.pop('anchor', tk.CENTER),
            'padx': kwargs.pop('padx', 10),  # 按钮默认水平间距10像素
            'pady': kwargs.pop('pady', 10)}  # 按钮默认垂直间距10像素
        kwargs['master'] = master if master else self.root

        # 创建组件
        image = kwargs.pop('image', None)  
        button = tk.Button(**kwargs)
        if image is not None:  # 配置图片
            self.config(button, image=image)
        button.pack(pack_kwargs)  # 放置组件

        return button
    
    def config(self, widget, **kwargs):
        """
        1. 仅当传入image参数时，才处理图片转换和引用
        2. 不传入image时，不修改image参数
        """
        # 处理图片参数（仅当传入image时）
        if 'image' in kwargs:
            img = kwargs['image']
            img_tk = self._convert_image(img)
            widget.img_tk = img_tk  # 关键：保留引用，避免垃圾回收
            kwargs['image'] = img_tk
        
        # 执行组件配置
        widget.config(**kwargs)

    def _convert_image(self, img):
        """
        将不同格式的图片转换为tkinter可用的PhotoImage对象
        支持：字符串路径、OpenCV图像（BGR格式）、PIL Image对象
        """
        if isinstance(img, str) and img:  # 字符串路径
            # 检查文件是否存在
            if not os.path.exists(img):
                raise FileNotFoundError(f"图片文件不存在：{img}")
            img_cv = cv2.imread(img)
            img_rgb = cv2.cvtColor(img_cv, cv2.COLOR_BGR2RGB)
            img_pil = Image.fromarray(img_rgb)
            img_tk = ImageTk.PhotoImage(img_pil)
        elif isinstance(img, np.ndarray):  # OpenCV图像（BGR格式）
            if len(img.shape) not in (2, 3):
                raise ValueError("OpenCV图像维度错误，需为2D/3D数组")
            img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB) if len(img.shape) == 3 else img
            img_pil = Image.fromarray(img_rgb)
            img_tk = ImageTk.PhotoImage(img_pil)
        elif isinstance(img, Image.Image):  # PIL Image对象
            img_tk = ImageTk.PhotoImage(img)
        elif isinstance(img, ImageTk.PhotoImage):
            img_tk = img
        else:  # 清空图片
            img_tk = ""  # 用空字符串替代None，避免tkinter报错
        return img_tk

    def after(self, delay, func):
        self.root.after(delay, func)

    def run(self):
        self.root.mainloop()

    def quit(self):
        self.root.quit()
        self.root.destroy()




import time
import threading
import numpy as np
import cv2
import netifaces
import subprocess
from unitree_sdk2py.core.channel import ChannelSubscriber, ChannelFactoryInitialize
from unitree_sdk2py.go2.sport.sport_client import SportClient
from unitree_sdk2py.go2.video.video_client import VideoClient
from unitree_sdk2py.idl.unitree_go.msg.dds_ import SportModeState_


error_code = {
    100: "灵动",
    1001: "阻尼",
    1002: "站立锁定",
    1004: "蹲下",
    2006: "蹲下",
    1006: "打招呼/伸懒腰/舞蹈/拜年/比心/开心",
    1007: "坐下",
    1008: "前跳",
    1009: "扑人",
    1013: "平衡站立",
    1015: "常规行走",
    1016: "常规跑步",
    1017: "常规续航",
    1091: "摆姿势",
    2004: "翻身",   # ???
    2007: "闪避",
    2008: "并腿跑",
    2009: "跳跃跑",
    2010: "经典",
    2011: "倒立",
    2012: "前空翻",
    2013: "后空翻",
    2014: "左空翻",
    2016: "交叉步",
    2017: "直立",
    2019: "牵引",
}


class Go2:
    """
    宇树Go2 运动控制封装类
    适用于：Unitree SDK2 Python接口
    """
    def __init__(self, interface=None, timeout=20.0):
        """
        初始化控制器
        :param interface: 网卡接口名称
        :param timeout: 超时时间（秒）
        """
        self.interface = interface
        self.timeout = timeout
        self.sport_client = None
        self.video_client = None
        self.cap = None
        self.error_code = 0  # 状态码
        
        # 移动控制相关变量
        self._moving = False
        self._move_params = (0, 0, 0)
        self._move_thread = None
        
        # 摄像头对象
        self.camera = None
        
        # 声光控制对象
        self._vui = None

        if self.interface is None:
            self.get_interface()
        
        # 清理标志
        self._initialized = False

    def get_interface(self):
        """获取当前接口"""

        interfaces = netifaces.interfaces()
        for iface in interfaces:
            if iface.startswith("en"):
                self.interface = iface
                break

    def check_go2_connection(self):
        """检查机器狗IP连通性"""
        try:
            # 先尝试不使用sudo ping
            result = subprocess.run(['ping', '-c', '1', '-W', '2', '192.168.123.161'], 
                                  capture_output=True, text=True, timeout=5)
            if result.returncode == 0:
                return True
            
            # 如果不使用sudo失败，尝试使用sudo
            result = subprocess.run(['sudo', 'ping', '-c', '1', '-W', '2', '192.168.123.161'], 
                                  capture_output=True, text=True, timeout=5)
            return result.returncode == 0
        except subprocess.TimeoutExpired:
            print("ping超时")
            return False
        except Exception as e:
            print(f"检查连接时出错: {e}")
            return False
        



    def init(self):
        """初始化与Go2的连接"""
        
        # 检查机器狗IP连通性（仅作为警告，不阻止初始化）
        if not self.check_go2_connection():
            print("警告：无法ping通机器狗IP，但继续尝试初始化SDK")
            print("请确保机器狗已开机且网络连接正常")
        
        try:
            ChannelFactoryInitialize(0, self.interface)

            # 启动状态订阅
            self.sub_state(self.callback)

            # 初始化运动控制客户端
            self.sport_client = SportClient()
            self.sport_client.SetTimeout(self.timeout)
            self.sport_client.Init()
            
            # 注意：视频流不再自动初始化，按需开启
            # self.cap = self.open_video()  # 移除自动初始化
            
            self._initialized = True
            print("Go2 SDK初始化成功")
            return True
        except Exception as e:
            print(f"SDK初始化失败: {e}")
            return False
        

    def open_video(self, width: int = 480, height: int = 320):
        """打开视频流"""
        gstreamer_str = (
            f"udpsrc address=230.1.1.1 port=1720 multicast-iface={self.interface} "
            "! application/x-rtp, media=video, encoding-name=H264 "
            "! rtph264depay ! h264parse "
            "! avdec_h264 "  # 解码H.264
            "! videoscale "  # 添加缩放元素，用于调整分辨率
            f"! video/x-raw,width={width},height={height} "  # 目标分辨率
            "! videoconvert ! video/x-raw, format=BGR "  # 转换为OpenCV支持的BGR格式
            "! appsink drop=1"
        )
        cap = cv2.VideoCapture(gstreamer_str, cv2.CAP_GSTREAMER)
        if not cap.isOpened():
            print("视频流打开失败")
            print(f"使用的网络接口: {self.interface}")
            print("GStreamer字符串:", gstreamer_str)
            return None
        print("视频流打开成功")
        return cap

    def read_image(self):
        """从视频流获取一帧图像"""
        if self.cap is None:
            print("视频流未打开，请先调用 open_video()")
            return None
            
        ret, frame = self.cap.read()
        if not ret:
            print("读取图像失败")
            return None
        return frame

    
    def sub_state(self, callback, queue_size: int = 5):
        subscriber = ChannelSubscriber("rt/sportmodestate", SportModeState_)
        subscriber.Init(callback, queue_size)

    def callback(self, msg):
        self.error_code = msg.error_code


    # def read_image(self):
    #     """从视频流获取一帧图像"""
    #     code, data = self.video_client.GetImageSample()
    #     if code != 0 or data is None:
    #         print("获取图像样本失败，错误码:", code)
    #         return None
    #     image_data = np.frombuffer(bytes(data), dtype=np.uint8)
    #     image = cv2.imdecode(image_data, cv2.IMREAD_COLOR)
    #     return image

    def Damp(self):
        """进入阻尼状态。"""
        self._call(self.sport_client.Damp)

    def BalanceStand(self):
        """解除锁定。"""
        self._call(self.sport_client.BalanceStand)

    def StopMove(self):
        """
        停止机器狗的所有移动动作，并重置相关状态。
        
        该方法会：
        1. 停止任何正在执行的移动线程
        2. 调用底层SDK的停止方法
        3. 重置移动状态标志
        
        注意:
        - 该方法会阻塞等待移动线程结束，最多等待1秒
        - 调用后所有运动指令将被重置为默认状态
        """
        print("停止移动")
        # 停止持续移动线程
        self._moving = False
        if (hasattr(self, '_move_thread') and 
            self._move_thread is not None and 
            self._move_thread.is_alive()):
            self._move_thread.join(timeout=1.0)
            self._move_thread = None
        
        # 调用SDK的停止方法
        self._call(self.sport_client.StopMove)

    def _call(self, func, *args, **kwargs):
        """在线程中调用运动控制函数"""
        def fun_thread():
            ret = func(*args, **kwargs)
            print(f"{func.__name__} 执行结果:", ret)
        
        t = threading.Thread(target=fun_thread)
        t.start()
        t.join()  # 等待线程完成



    def StandUp(self):
        """
        关节锁定，站高。
        执行后状态: 1002:站立锁定
        """
        # 执行前判断状态
        if self.error_code in [1002]:  # 1002 :站立锁定
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1001, 1007, 1013]:  # 100 :灵动, 1001 :阻尼, 1013 :平衡站立
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        self._call(self.sport_client.StandUp)



    def StandDown(self):
        """
        关节锁定，站低。
        执行后状态: 1001:阻尼
        """
        # 执行前判断状态
        if self.error_code in [1001, 1004, 2006]:  # 1001:阻尼 1004 :蹲下 2006 :蹲下
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 100 :灵动, 1001 :阻尼, 1013 :平衡站立
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.StandDown)

    def RecoveryStand(self):
        """	恢复站立。"""
        self._call(self.sport_client.RecoveryStand)

    def Euler(self, roll, pitch, yaw):
        """站立和行走时的姿态。"""
        pass

    def Move(self, vx, vy, vyaw):
        """移动。"""
        # 检查状态是否允许移动
        if self.error_code not in [100, 1002, 1013]:  # 100:灵动, 1002:站立锁定, 1013:平衡站立
            print(f"当前状态不允许移动: {self.error_code} - {error_code.get(self.error_code, '未知状态')}")
            return False
        
        # 限制速度范围
        vx = max(-1.0, min(1.0, vx))  # 前后速度限制在-1到1之间
        vy = max(-1.0, min(1.0, vy))  # 左右速度限制在-1到1之间
        vyaw = max(-2.0, min(2.0, vyaw))  # 转动速度限制在-2到2之间
        
        def move_thread():
            ret = self.sport_client.Move(vx, vy, vyaw)
            success = ret == 0
            print(f"Move 执行结果: {ret}, 成功: {success}")
            return success
        
        t = threading.Thread(target=move_thread)
        t.start()
        t.join()
        return True

    def MoveForDuration(self, vx, vy, vyaw, duration):
        """
        持续移动指定时间
        
        Args:
            vx (float): 前后速度 (-1.0 到 1.0)
            vy (float): 左右速度 (-1.0 到 1.0)  
            vyaw (float): 转动速度 (-2.0 到 2.0)
            duration (float): 移动时间（秒）
            
        Returns:
            bool: 是否成功执行
        """
        print(f"开始移动: vx={vx}, vy={vy}, vyaw={vyaw}, 持续时间={duration}秒")
        start_time = time.time()
        
        try:
            while time.time() - start_time < duration:
                if not self.Move(vx, vy, vyaw):
                    return False
                time.sleep(0.1)  # 每100ms调用一次
            
            print("移动完成")
            return True
            
        except KeyboardInterrupt:
            print("移动被用户中断")
            return False

    def Forward(self, speed=0.3, duration=2.0):
        """向前移动"""
        return self.MoveForDuration(speed, 0, 0, duration)

    def Backward(self, speed=0.3, duration=2.0):
        """向后移动"""
        return self.MoveForDuration(-speed, 0, 0, duration)

    def Left(self, speed=0.3, duration=2.0):
        """向左移动"""
        return self.MoveForDuration(0, speed, 0, duration)

    def Right(self, speed=0.3, duration=2.0):
        """向右移动"""
        return self.MoveForDuration(0, -speed, 0, duration)

    def TurnLeft(self, speed=0.5, duration=2.0):
        """左转"""
        return self.MoveForDuration(0, 0, speed, duration)

    def TurnRight(self, speed=0.5, duration=2.0):
        """右转"""
        return self.MoveForDuration(0, 0, -speed, duration)

    def StartMove(self, vx, vy, vyaw):
        """
        开始持续移动，需要调用StopMove来停止
        
        Args:
            vx (float): 前后速度 (-1.0 到 1.0)
            vy (float): 左右速度 (-1.0 到 1.0)  
            vyaw (float): 转动速度 (-2.0 到 2.0)
            
        Returns:
            bool: 是否成功开始移动
        """
        print(f"开始持续移动: vx={vx}, vy={vy}, vyaw={vyaw}")
        
        # 先检查是否可以移动
        if not self.Move(vx, vy, vyaw):
            print("无法开始持续移动：当前状态不允许移动")
            return False
            
        self._moving = True
        self._move_params = (vx, vy, vyaw)
        
        def continuous_move():
            while self._moving:
                if not self.Move(vx, vy, vyaw):
                    print("持续移动失败，停止移动")
                    break
                time.sleep(0.1)
        
        self._move_thread = threading.Thread(target=continuous_move)
        self._move_thread.daemon = True  # 设置为守护线程
        self._move_thread.start()
        return True

    def StartForward(self, speed=0.3):
        """开始向前移动"""
        return self.StartMove(speed, 0, 0)

    def StartBackward(self, speed=0.3):
        """开始向后移动"""
        return self.StartMove(-speed, 0, 0)

    def StartLeft(self, speed=0.3):
        """开始向左移动"""
        return self.StartMove(0, speed, 0)

    def StartRight(self, speed=0.3):
        """开始向右移动"""
        return self.StartMove(0, -speed, 0)

    def StartTurnLeft(self, speed=0.5):
        """开始左转"""
        return self.StartMove(0, 0, speed)

    def StartTurnRight(self, speed=0.5):
        """开始右转"""
        return self.StartMove(0, 0, -speed)

    def Sit(self):
        """
        坐下。
        执行后状态: 1007: 坐下
        """
        # 执行前判断状态
        if self.error_code in [1007]:  # 1007 : 坐下
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 100 :灵动, 1001 :阻尼, 1013 :平衡站立
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Sit)

    def RiseSit(self):
        """
        站起（相对于坐下）。
        执行后状态: 
        """
        # 执行前判断状态
        if self.error_code in [1007]:  # 1002 :站立锁定
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1007, 1013]:  # 100 :灵动, 1007 : 坐下, 1013 :平衡站立
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.RiseSit)



    def SpeedLevel(self, level: int):
        """设置速度档位。"""
        pass

    def Hello(self):
        """
        打招呼
        执行后状态: 1013:平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return

        # 执行指令
        self._call(self.sport_client.Hello)

    def Stretch(self):
        """
        伸懒腰。
        执行后状态: 1013:平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 站立且空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Stretch)



    def Content(self):
        """
        开心。
        执行后状态: 1013:平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Content)


    def Heart(self):
        """
        比心。
        执行后状态: 1013:平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Heart)


    def Pose(self, flag):
        """摆姿势。"""
        pass

    def Scrape(self):
        """ 
        拜年作揖。
        执行后状态: 1013:平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Scrape)



    def FrontJump(self):
        """ 
        前跳。
        执行后状态: 1013: 平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1008]:  # 1008 : 前跳
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.FrontJump)



    def FrontPounce(self):
        """ 
        向前扑人。
        执行后状态: 1013: 平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1009]:  # 1009 : 扑人
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.FrontPounce)


    def Dance1(self):
        """
        舞蹈段落1。
        执行后状态: 1013: 平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Dance1)
    
    def Dance2(self):
        """
        舞蹈段落2。
        执行后状态: 1013: 平衡站立
        """
        # 执行前判断状态
        if self.error_code in [1006]:  # 1006 :打招呼/伸懒腰/舞蹈/拜年/比心/开心
            print("当前状态:", self.error_code, error_code[self.error_code], "无需执行")
            return
        if self.error_code not in [100, 1002, 1013]:  # 空闲状态
            print("繁忙中,当前状态:", self.error_code, error_code[self.error_code])
            return
        
        # 执行指令
        self._call(self.sport_client.Dance2)
    
    def HandStand(self, flag: int):
        """倒立行走。"""
        # 执行指令
        self._call(lambda: self.sport_client.HandStand(bool(flag)))
        if flag:
            time.sleep(4)
            self._call(lambda: self.sport_client.HandStand(False))

    def LeftFlip(self):
        """左空翻。"""
        # 执行指令
        result = self._call(self.sport_client.LeftFlip)
        return result == 0 if result is not None else False

    def BackFlip(self):
        """后空翻。"""
        # 执行指令
        result = self._call(self.sport_client.BackFlip)
        return result == 0 if result is not None else False

    def FreeWalk(self, flag: int = None):
        """ 灵动模式（默认步态）。"""
        # 执行指令
        if flag is None:
            # 切换模式
            result = self._call(lambda: self.sport_client.FreeWalk())
        else:
            # 根据flag值执行相应操作
            if flag:
                # 开启灵动模式
                result = self._call(lambda: self.sport_client.FreeWalk())
            else:
                # 关闭灵动模式 - 切换到其他模式
                result = self._call(lambda: self.sport_client.StandUp())
        return result == 0 if result is not None else False

    def FreeBound(self, flag: int):
        """ 并腿跑模式。"""
        # 执行指令
        result = self._call(lambda: self.sport_client.FreeBound(bool(flag)))
        success = result == 0 if result is not None else False
        
        if flag and success:
            # 如果成功开启，等待2秒后自动关闭（这是为了演示效果）
            time.sleep(2)
            self._call(lambda: self.sport_client.FreeBound(False))
        
        return success

    def FreeJump(self, flag: int):
        """ 跳跃模式。"""
        # 执行指令
        result = self._call(lambda: self.sport_client.FreeJump(bool(flag)))
        success = result == 0 if result is not None else False
        
        if flag and success:
            # 如果成功开启，等待4秒后自动关闭（这是为了演示效果）
            time.sleep(4)
            self._call(lambda: self.sport_client.FreeJump(False))
        
        return success

    def FreeAvoid(self, flag: int):
        """ 闪避模式。"""
        # 执行指令
        result = self._call(lambda: self.sport_client.FreeAvoid(bool(flag)))
        success = result == 0 if result is not None else False
        
        if flag and success:
            # 如果成功开启，等待2秒后自动关闭（这是为了演示效果）
            time.sleep(2)
            self._call(lambda: self.sport_client.FreeAvoid(False))
        
        return success

    def WalkUpright(self, flag: int):
        """ 后腿直立模式。"""
        # 执行指令
        result = self._call(lambda: self.sport_client.WalkUpright(bool(flag)))
        success = result == 0 if result is not None else False
        
        if flag and success:
            # 如果成功开启，等待4秒后自动关闭（这是为了演示效果）
            time.sleep(4)
            self._call(lambda: self.sport_client.WalkUpright(False))
        
        return success

    def CrossStep(self, flag: int):
        """ 交叉步模式。"""
        # 执行指令
        result = self._call(lambda: self.sport_client.CrossStep(bool(flag)))
        success = result == 0 if result is not None else False
        
        if flag and success:
            # 如果成功开启，等待4秒后自动关闭（这是为了演示效果）
            time.sleep(4)
            self._call(lambda: self.sport_client.CrossStep(False))
        
        return success

    def AutoRecoverSet(self, flag: int):
        """ 设置自动翻身是否生效。"""
        try:
            # 执行指令
            result = self._call(lambda: self.sport_client.AutoRecoverSet(bool(flag)))
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: AutoRecoverSet 方法在当前SDK版本中不可用")
            return False

    def AutoRecoverGet(self):
        """ 查询自动翻身是否生效。"""
        try:
            # 执行指令
            result = self._call(self.sport_client.AutoRecoverGet)
            return result if result is not None else False
        except AttributeError:
            print("警告: AutoRecoverGet 方法在当前SDK版本中不可用")
            return False

    def ClassicWalk(self, flag: int):
        """ 经典步态。"""
        try:
            # 执行指令
            result = self._call(lambda: self.sport_client.ClassicWalk(bool(flag)))
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: ClassicWalk 方法在当前SDK版本中不可用")
            return False

    def TrotRun(self):
        """ 进入常规跑步模式 """
        try:
            # 执行指令
            result = self._call(self.sport_client.TrotRun)
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: TrotRun 方法在当前SDK版本中不可用")
            return False

    def StaticWalk(self):
        """ 进入常规行走模式"""
        try:
            # 执行指令
            result = self._call(self.sport_client.StaticWalk)
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: StaticWalk 方法在当前SDK版本中不可用")
            return False

    def EconomicGait(self):
        """ 进入常规续航模式 """
        try:
            # 执行指令
            result = self._call(self.sport_client.EconomicGait)
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: EconomicGait 方法在当前SDK版本中不可用")
            return False

    def SwitchAvoidMode(self):
        """ 闪避模式下，关闭摇杆未推时前方障碍物的闪避以及后方的障碍物躲避"""
        try:
            # 执行指令
            result = self._call(self.sport_client.SwitchAvoidMode)
            return result == 0 if result is not None else False
        except AttributeError:
            print("警告: SwitchAvoidMode 方法在当前SDK版本中不可用")
            return False
    
    def get_camera(self):
        """
        获取摄像头对象（按需初始化）
        
        Returns:
            Go2Camera: 摄像头控制对象
        """
        if self.camera is None:
            # 直接导入go2_camera模块
            from .go2_camera import Go2Camera
            self.camera = Go2Camera(self.interface, self.timeout)
            # 注意：这里不自动初始化，让用户按需调用
        return self.camera
    
    def get_vui(self):
        """
        获取声光控制对象（按需初始化）
        
        Returns:  
            Go2VUI: 声光控制对象
        """
        if not hasattr(self, '_vui') or self._vui is None:
            # 直接导入go2_vui模块
            from .go2_vui import Go2VUI
            self._vui = Go2VUI(self.interface)
            # 注意：这里不自动初始化，让用户按需调用
        return self._vui
    
    def capture_image(self, save_path=None):
        """
        便利方法：获取一张图片（按需初始化摄像头）
        
        Args:
            save_path (str): 保存路径
            
        Returns:
            numpy.ndarray: 图像数据
        """
        camera = self.get_camera()
        if not camera.init(self.interface):
            print("摄像头初始化失败")
            return None
        return camera.capture_image(save_path)
    
    def start_video_stream(self, width=480, height=320):
        """
        便利方法：开始视频流（按需初始化摄像头）
        
        Args:
            width (int): 视频宽度
            height (int): 视频高度
            
        Returns:
            bool: 是否成功
        """
        camera = self.get_camera()
        if not camera.init(self.interface):
            print("摄像头初始化失败")
            return False
        return camera.start_stream(width, height)
    
    def get_video_frame(self):
        """
        便利方法：获取最新视频帧
        
        Returns:
            numpy.ndarray: 图像数据
        """
        if self.camera is None:
            print("视频流未启动，请先调用 start_video_stream()")
            return None
        return self.camera.get_latest_frame()
    
    def stop_video_stream(self):
        """便利方法：停止视频流"""
        if self.camera:
            self.camera.stop_stream()
    
    def cleanup(self):
        """清理资源，停止所有连接和线程"""
        if not self._initialized:
            return
            
        print("正在清理Go2资源...")
        
        # 停止移动线程
        self._moving = False
        if (hasattr(self, '_move_thread') and 
            self._move_thread is not None and 
            self._move_thread.is_alive()):
            self._move_thread.join(timeout=1.0)
            self._move_thread = None
        
        # 清理摄像头资源
        if self.camera is not None:
            try:
                self.camera.cleanup()
                self.camera = None
            except:
                pass
        
        # 释放视频流
        if self.cap is not None:
            try:
                self.cap.release()
                self.cap = None
            except:
                pass
        
        # 清理DDS连接
        try:
            from unitree_sdk2py.core.channel import ChannelFactoryRelease
            ChannelFactoryRelease()
        except:
            pass
        
        self._initialized = False
        print("Go2资源清理完成")
    
    def __del__(self):
        """析构函数，自动清理资源"""
        try:
            self.cleanup()
        except:
            pass




# -------------------- 测试 --------------------
if __name__ == "__main__":
    interface = "enx00e0986113a6"  # 替换为你的Go2网卡接口名称
    
    go2 = Go2(interface=interface)

    go2.stand_down()
    go2.stand_up()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
ezgo - 宇树Go2机器狗Python控制库

这是一个用于控制宇树Go2机器狗的Python库，提供了简单易用的API接口。
支持运动控制、视频流获取、UI界面等功能。
"""

__version__ = "0.0.9"
__author__ = "ezgo"
__email__ = ""
__license__ = "MIT"

# 懒加载模块 - 只在实际使用时才导入
def _lazy_import(module_name, class_name):
    """懒加载模块，避免导入时的依赖检查"""
    def _import():
        try:
            module = __import__(f'.{module_name}', package=__name__, fromlist=[class_name])
            return getattr(module, class_name)
        except ImportError as e:
            # 只在使用时才提示依赖缺失
            raise ImportError(f"使用 {class_name} 需要安装额外依赖: {e}")
    return _import

# 定义懒加载函数
_Go2 = _lazy_import('go2', 'Go2')
_Camera = _lazy_import('camera', 'Camera')
_APP = _lazy_import('ui', 'APP')
_Go2Camera = _lazy_import('go2_camera', 'Go2Camera')
_Go2VUI = _lazy_import('go2_vui', 'Go2VUI')

# 直接导入不需要额外依赖的模块
try:
    from .eztk import EasyTk
    from .ezcamera import Camera as EzCamera
except ImportError as e:
    # 这些模块应该能正常导入，如果失败则显示错误
    print(f"错误: 无法导入核心模块: {e}")
    EasyTk = None
    EzCamera = None

# 定义公开的API
__all__ = [
    "Go2",
    "Camera", 
    "APP",
    "Go2Camera",
    "Go2VUI",
    "EasyTk",
    "EzCamera",
    "__version__"
]

# 懒加载属性访问器
class _LazyModule:
    def __init__(self, import_func):
        self._import_func = import_func
        self._module = None
    
    def __call__(self, *args, **kwargs):
        if self._module is None:
            self._module = self._import_func()
        return self._module(*args, **kwargs)
    
    def __getattr__(self, name):
        if self._module is None:
            self._module = self._import_func()
        return getattr(self._module, name)

# 创建懒加载对象
Go2 = _LazyModule(_Go2)
Camera = _LazyModule(_Camera)
APP = _LazyModule(_APP)
Go2Camera = _LazyModule(_Go2Camera)
Go2VUI = _LazyModule(_Go2VUI)




"""
Go2机器狗摄像头控制类
提供图片获取和视频流功能
"""

import cv2
import numpy as np
import threading
import time
from unitree_sdk2py.core.channel import ChannelFactoryInitialize
from unitree_sdk2py.go2.video.video_client import VideoClient


class Go2Camera:
    """
    Go2机器狗摄像头控制类
    支持单张图片获取和视频流获取
    """
    
    def __init__(self, interface=None, timeout=3.0):
        """
        初始化摄像头控制器
        
        Args:
            interface (str): 网卡接口名称
            timeout (float): 超时时间（秒）
        """
        self.interface = interface
        self.timeout = timeout
        self.video_client = None
        self.cap = None
        self._streaming = False
        self._stream_thread = None
        self._latest_frame = None
        self._frame_lock = threading.Lock()
        
    def init(self, interface=None):
        """
        初始化摄像头连接
        
        Args:
            interface (str): 网卡接口名称，如果为None则使用初始化时的接口
            
        Returns:
            bool: 是否初始化成功
        """
        if interface:
            self.interface = interface
            
        if not self.interface:
            print("错误：未指定网络接口")
            return False
            
        try:
            # 初始化DDS通道（与go2.py保持一致）
            ChannelFactoryInitialize(0, self.interface)
            
            # 初始化视频客户端
            self.video_client = VideoClient()
            self.video_client.SetTimeout(self.timeout)
            self.video_client.Init()
            
            print(f"摄像头初始化成功，使用接口: {self.interface}")
            return True
            
        except Exception as e:
            print(f"摄像头初始化失败: {e}")
            return False
    
    def capture_image(self, save_path=None):
        """
        获取一张图片
        
        Args:
            save_path (str): 保存路径，如果为None则不保存
            
        Returns:
            numpy.ndarray: 图像数据，失败时返回None
        """
        if not self.video_client:
            print("错误：摄像头未初始化，请先调用 init()")
            return None
            
        try:
            # 获取图像样本
            code, data = self.video_client.GetImageSample()
            
            if code != 0 or data is None:
                print(f"获取图像失败，错误码: {code}")
                return None
                
            # 转换图像数据
            image_data = np.frombuffer(bytes(data), dtype=np.uint8)
            image = cv2.imdecode(image_data, cv2.IMREAD_COLOR)
            
            if image is None:
                print("图像解码失败")
                return None
                
            print(f"成功获取图像，尺寸: {image.shape}")
            
            # 保存图像
            if save_path:
                cv2.imwrite(save_path, image)
                print(f"图像已保存到: {save_path}")
                
            return image
            
        except Exception as e:
            print(f"获取图像时出错: {e}")
            return None
    
    def open_video_stream(self, width=480, height=320):
        """
        打开视频流
        
        Args:
            width (int): 视频宽度
            height (int): 视频高度
            
        Returns:
            bool: 是否成功打开视频流
        """
        if not self.interface:
            print("错误：未指定网络接口")
            return False
            
        # 构建GStreamer字符串
        gstreamer_str = (
            f"udpsrc address=230.1.1.1 port=1720 multicast-iface={self.interface} "
            "! application/x-rtp, media=video, encoding-name=H264 "
            "! rtph264depay ! h264parse "
            "! avdec_h264 "  # 解码H.264
            "! videoscale "  # 添加缩放元素，用于调整分辨率
            f"! video/x-raw,width={width},height={height} "  # 目标分辨率
            "! videoconvert ! video/x-raw, format=BGR "  # 转换为OpenCV支持的BGR格式
            "! appsink drop=1"
        )
        
        try:
            self.cap = cv2.VideoCapture(gstreamer_str, cv2.CAP_GSTREAMER)
            
            if not self.cap.isOpened():
                print("视频流打开失败")
                print(f"使用的网络接口: {self.interface}")
                print("GStreamer字符串:", gstreamer_str)
                return False
                
            print("视频流打开成功")
            return True
            
        except Exception as e:
            print(f"打开视频流时出错: {e}")
            return False
    
    def read_frame(self):
        """
        从视频流读取一帧
        
        Returns:
            numpy.ndarray: 图像数据，失败时返回None
        """
        if self.cap is None or not self.cap.isOpened():
            print("错误：视频流未打开")
            return None
            
        try:
            ret, frame = self.cap.read()
            if not ret:
                print("读取视频帧失败")
                return None
                
            return frame
            
        except Exception as e:
            print(f"读取视频帧时出错: {e}")
            return None
    
    def start_stream(self, width=480, height=320):
        """
        开始后台视频流（持续获取最新帧）
        
        Args:
            width (int): 视频宽度
            height (int): 视频高度
            
        Returns:
            bool: 是否成功开始流
        """
        if self._streaming:
            print("视频流已在运行中")
            return True
            
        if not self.open_video_stream(width, height):
            return False
            
        self._streaming = True
        
        def stream_loop():
            while self._streaming:
                frame = self.read_frame()
                if frame is not None:
                    with self._frame_lock:
                        self._latest_frame = frame.copy()
                time.sleep(0.01)  # 避免过度占用CPU
                
        self._stream_thread = threading.Thread(target=stream_loop)
        self._stream_thread.daemon = True
        self._stream_thread.start()
        
        print("后台视频流已启动")
        return True
    
    def get_latest_frame(self):
        """
        获取最新的视频帧（非阻塞）
        
        Returns:
            numpy.ndarray: 最新图像数据，无数据时返回None
        """
        with self._frame_lock:
            if self._latest_frame is not None:
                return self._latest_frame.copy()
            return None
    
    def stop_stream(self):
        """停止后台视频流"""
        self._streaming = False
        if self._stream_thread and self._stream_thread.is_alive():
            self._stream_thread.join(timeout=1.0)
        print("后台视频流已停止")
    
    def close_video_stream(self):
        """关闭视频流"""
        self.stop_stream()
        if self.cap is not None:
            self.cap.release()
            self.cap = None
            print("视频流已关闭")
    
    def cleanup(self):
        """清理所有资源"""
        self.stop_stream()
        self.close_video_stream()
        self.video_client = None
        print("摄像头资源清理完成")
    
    def __del__(self):
        """析构函数"""
        try:
            self.cleanup()
        except:
            pass


# 便利函数
def capture_single_image(interface=None, save_path=None):
    """
    便利函数：获取单张图片
    
    Args:
        interface (str): 网卡接口
        save_path (str): 保存路径
        
    Returns:
        numpy.ndarray: 图像数据
    """
    camera = Go2Camera()
    if camera.init(interface):
        return camera.capture_image(save_path)
    return None


def create_video_stream(interface=None, width=480, height=320):
    """
    便利函数：创建视频流对象
    
    Args:
        interface (str): 网卡接口
        width (int): 视频宽度
        height (int): 视频高度
        
    Returns:
        Go2Camera: 摄像头对象
    """
    camera = Go2Camera(interface)
    if camera.init(interface):
        if camera.open_video_stream(width, height):
            return camera
    return None


import tkinter as tk
import platform
import os


if platform.system() == "Linux":
    os.environ["DISPLAY"] = ":0"

class APP(tk.Tk):
    def __init__(self, title="APP", width=480, height=800):
        super().__init__()
        self.title_text = title
        self.title(self.title_text)
        self.geometry(f"{width}x{height}")
        
        if platform.system() == "Linux":
            self.resizable(False, False)  # 固定窗口大小
            self.attributes('-fullscreen', True)  # 全屏
            self.config(cursor="none")  # 隐藏鼠标
            self.update_idletasks()  # 强制立即应用全屏，避免闪烁/延迟
        
        # 显示图像和文字的全局变量
        self.image = None  # 显示的图像
        self.text = ""  # 显示的文字

        self.is_running = True  # 程序运行状态

        # 初始化组件
        self.create_widgets()
        # 绑定窗口关闭事件
        self.protocol("WM_DELETE_WINDOW", self.on_close)

    def create_widgets(self):
        # 标题
        title_label = tk.Label(self, text=self.title_text, font=("SimHei", 18))
        title_label.pack(pady=5)

        # 图像显示区域
        self.frame1 = tk.LabelFrame(self, text="图像", width=480, height=480, font=("SimHei", 10))
        self.frame1.pack_propagate(False)
        self.frame1.pack(fill=tk.X, pady=5)
        self.image_label = tk.Label(self.frame1)
        self.image_label.pack(pady=5, fill=tk.BOTH, expand=True, anchor=tk.CENTER)

        # 文字显示区域
        self.frame2 = tk.LabelFrame(self, text="结果", width=480, height=200, font=("SimHei", 10))
        self.frame2.pack_propagate(False)
        self.frame2.pack(fill=tk.X, pady=5)
        self.text_label = tk.Label(self.frame2, 
                                   text="文本显示", 
                                   font=("SimHei", 14),
                                   wraplength=450,
                                   justify=tk.CENTER,)
        self.text_label.pack(pady=5, fill=tk.BOTH, expand=True, anchor=tk.CENTER)


        # 退出按钮
        self.exit_btn = tk.Button(self, text="退出", command=self.on_close, font=("SimHei", 16))
        self.exit_btn.pack(pady=10)

    
    def set_image(self, image):
        """设置显示的图像"""
        self.image = image
        self.image_label.config(image=self.image)

    def set_text(self, text):
        """设置显示的文字"""
        self.text = text
        self.text_label.config(text=self.text)

    def on_close(self):
        """窗口关闭/退出按钮回调"""
        self.is_running = False
        self.quit()  # 退出主循环
        self.destroy()  # 销毁窗口




import cv2
from PIL import Image, ImageTk
import platform
import time
import queue
import threading


class Camera:
    def __init__(self, index=0, width=640, height=480, fps=30,):
        """
        初始化摄像头对象。

        参数:
        - index: 摄像头索引，默认值为 0。
        - width: 视频宽度，默认值为 640。
        - height: 视频高度，默认值为 480。
        - fps: 视频帧率，默认值为 30。
        """
        # 基础参数
        self.index = index
        self.width = width
        self.height = height
        self.fps = fps

        # 状态控制
        self.is_opened = False  # 摄像头是否成功打开
        self.is_running = False  # 线程退出标志
        self.first_frame_ready = threading.Event()  # 第一帧就绪事件

        # 资源管理
        self.cap = None
        self.capture_thread = None  # 读帧线程对象


        # 双缓冲：两个缓冲区
        self.buffer_write = None  # 写入缓冲区
        self.buffer_read = None   # 读取缓冲区
        self.buffer_lock = threading.Lock()  # 用于交换缓冲区的锁

        # 线程锁
        # self.lock = threading.Lock()
        
        

    def open(self, timeout=5):
        """
        打开摄像头。
        若在 Linux 系统下，启用 V4L2 硬件加速和 MJPG 压缩格式。

        参数:
        - timeout: 等待第一帧图像的超时时间，默认值为 5 秒。
        """
        
        if self.is_opened:
            print(f"摄像头 {self.index} 已打开")
            return True

        if platform.system() == "Linux":
            self.cap = cv2.VideoCapture(self.index, cv2.CAP_V4L2)
            # 尝试设置MJPG格式（硬件加速）
            fourcc = cv2.VideoWriter_fourcc(*'MJPG')
            self.cap.set(cv2.CAP_PROP_FOURCC, fourcc)
            self.cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)  # 减少摄像头缓冲区，降低延迟
        elif platform.system() == "Windows":
            self.cap = cv2.VideoCapture(self.index, cv2.CAP_DSHOW)  # 替代默认的CAP_MSMF，降低延迟
            fourcc = cv2.VideoWriter_fourcc(*'MJPG')
            self.cap.set(cv2.CAP_PROP_FOURCC, fourcc)
        else:  # macOS
            self.cap = cv2.VideoCapture(self.index)
            self.cap.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc(*'MJPG'))

        if not self.cap.isOpened():
            raise Exception("无法打开摄像头")

        # 仅当实际分辨率不一致时才设置分辨率
        actual_width = int(self.cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        actual_height = int(self.cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        if actual_width != self.width or actual_height != self.height:
            self.cap.set(cv2.CAP_PROP_FRAME_WIDTH, self.width)
            self.cap.set(cv2.CAP_PROP_FRAME_HEIGHT, self.height)
        
        # 重置状态
        self.first_frame_ready.clear()
        self.buffer_write = None
        self.buffer_read = None

        # 启动读帧线程
        self.is_running = True
        self.capture_thread = threading.Thread(target=self._capture_thread, daemon=True)
        self.capture_thread.start()

        # 等待第一帧就绪（带超时）
        if not self.first_frame_ready.wait(timeout=timeout):
            self.close()
            raise TimeoutError(f"等待第一帧超时（{timeout}秒）")

        self.is_opened = True

        return self

    def _capture_thread(self):
        """读取图像帧线程"""
        frame_interval = 1.0 / self.fps  # 理论帧间隔
        first_frame = True
        next_frame_time = time.time()  # 初始化
        while self.is_running and self.cap.isOpened():
            start_time = time.time()
            # 读取帧
            ret, frame = self.cap.read()
            if not ret:
                time.sleep(0.001)
                continue

            # 写入缓冲区
            with self.buffer_lock:
                # 先交换缓冲区：把已写好的交给读缓冲区
                self.buffer_write, self.buffer_read = self.buffer_read, self.buffer_write
                # 再写入新帧到空的写缓冲区
                self.buffer_write = frame

            # 标记第一帧就绪
            if first_frame  and self.buffer_read is not None:
                self.first_frame_ready.set()
                first_frame = False
                
            # 基于绝对时间的帧率控制，避免累积误差
            next_frame_time += frame_interval
            sleep_time = max(0, next_frame_time - time.time())
            # 短睡眠用time.sleep，长睡眠用cv2.waitKey（更低CPU占用）
            if sleep_time > 0.01:
                cv2.waitKey(int(sleep_time * 1000))  # 毫秒级，释放GIL
            else:
                time.sleep(sleep_time)

    def read(self):
        """读取最新帧"""
        frame = None
        with self.buffer_lock:
            if self.buffer_read is not None:
                frame = self.buffer_read  # 先引用，不拷贝
        return frame.copy() if frame is not None else None  # 仅当有数据时拷贝
    
    def save_image(self, frame, filename="frame.jpg"):
        """保存当前帧到文件"""
        if frame is None:
            raise ValueError("无法保存空帧")
        cv2.imwrite(filename, frame)
        
    def resize(self, frame, size=(224, 224)):
        """将图像调整为指定尺寸"""
        if frame is None:
            return None
        return cv2.resize(frame, size, interpolation=cv2.INTER_LINEAR)
    
    def crop_to_square(self, frame):
        """将图像裁剪为正方形"""
        if frame is None:
            return None
        h, w = frame.shape[:2]
        min_dim = min(h, w)
        y = (h - min_dim) // 2
        x = (w - min_dim) // 2
        return frame[y:y+min_dim, x:x+min_dim]
    
    def close(self):
        """手动释放资源"""
        self.is_running = False  # 停止读帧线程
        self.is_opened = False

        if self.capture_thread is not None:
            self.capture_thread.join(timeout=1.0)  # 等待线程退出
            self.capture_thread = None
            
        if self.cap is not None:
            self.cap.release()
            self.cap = None
    
    def __del__(self):
        """释放资源"""
        self.close()